require 'rails_helper'

module ConfigManager
	describe Setting, :redis => true do
		let(:name) { "mysetting" }

		describe ".create" do
			subject { Setting.create(:id => id, :value => value, :tags => tags) }
			let(:id) { 'id' }
			let(:value) { 'value' }
			let(:tags) { "soccer, football, futbol" }

			context" when everything is good" do
				it "returns the created setting" do
					subject.id.should == id
					subject.value.should == value
					subject.tags.should include(*tags.split(', '))
				end
			end

			context "when id is missing" do
				let(:id) { nil }

				it "returns an invalid setting" do
					subject.should_not be_valid
				end

				it "doesn't persist the setting" do
					expect(Setting).to receive(:persist).never
					subject
				end
			end

			context "when id is blank" do
				let(:id) { "" }

				it "returns an invalid setting" do
					subject.should_not be_valid
				end

				it "doesn't persist the setting" do
					expect(Setting).to receive(:persist).never
					subject
				end
			end

			context "when value is missing" do
				let(:value) { nil }

				it "returns an invalid setting" do
					subject.should_not be_valid
				end

				it "doesn't persist the setting" do
					expect(Setting).to receive(:persist).never
					subject
				end
			end
		end

		describe ".persist" do
			let(:key) { Setting.send(:generate_key, name) }
			let(:global) { Setting.send(:global_key) }
			
			def redis_index(index)
				Setting.send(:generate_index, index)
			end

			it "stores the value in redis under the generated key" do
				Setting.persist(name, "I am serialized")
				$redis.get(key).should == "I am serialized"
			end

			it "adds the key to the global setting index" do
				Setting.persist(name, "I am serialized")
				$redis.smembers(global).should == [key]
			end

			context "when a tag  has been passed" do
				let(:tag) { "xyz" }

				it "adds the key to a set identified by the tag" do
					Setting.persist(name, "I am serialized", tag)
					$redis.smembers(redis_index(tag)).should include(key)
				end

				it "adds the key to the global setting index" do
					Setting.persist(name, "I am serialized", tag)
					$redis.smembers(global).should == [key]
				end
			end

			context "when two tags have been passed" do
				let(:tag1) { "xyz" }
				let(:tag2) { "abc" }

				it "adds the key to two sets identifed by the tags" do
					Setting.persist(name, "I am serialized", tag1, tag2)
					$redis.smembers(redis_index(tag1)).should include(key)
					$redis.smembers(redis_index(tag2)).should include(key)
				end
			end
		end

		describe ".find" do
			context "when id is not in redis" do
				it "returns nil" do
					Setting.find("mysetting").should be_nil
				end
			end

			context" when id is in redis" do
				before { Setting.persist(name, "I am serialized", "xyz", "abc") }
				subject { Setting.find(name, with_tags) }
				let(:with_tags) { false }

				it "returns a setting instance" do
					subject.should be_instance_of(Setting)
				end

				describe "returned instance" do
					it "has the right id" do
						subject.id.should == name
					end

					it "has the right value" do
						subject.value.should == "I am serialized"
					end
					
					context "when tags are not explicitely requested" do
						it "has no tags" do
							subject.tags.should be_empty
						end
					end

					context "when tags are requested" do
						let(:with_tags) { true }

						it "has the right tags" do
							["xyz", "abc"].each do |tag|
								subject.tags.should include(tag)
							end
						end
					end
				end
			end
		end

		describe ".value" do
			context "when setting for id doesn't exist" do
				it "returns nil" do
					Setting.value(name).should be_nil
				end
			end

			context "when setting for id exists" do
				before { Setting.persist("a", "A", "vowels") }

				it "returns its value" do
					Setting.value("a").should == "A"
				end
			end
		end

		describe ".all" do
			let(:settings) { Setting.all }

			context "when there are no settings defined" do
				it "returns an empty array" do
					settings.should be_empty
				end
			end

			context "when there are several settings defined" do
				before do
					Setting.persist("a", "A", "vowels")
					Setting.persist("b", "B", "consonants")
					Setting.persist("c", "C", "consonants")
				end

				it "returns an array of all setting instances" do
					settings.should have(3).items
					settings.each do |setting|
						setting.should be_instance_of(Setting)
					end
				end

				describe "returned settings array" do
					it "contains the settings with correct ids, values and tags" do
						settings.sort_by(&:id).map(&:id).should == ['a', 'b', 'c']
						settings.sort_by(&:id).map(&:value).should == ['A', 'B', 'C']
						settings.sort_by(&:id).map(&:tags).
								should == [["vowels"], ["consonants"], ["consonants"]]
					end
				end

				context "one of the settings has been deleted" do
					before { Setting.delete("b") }

					it "returns an array of remaining instances" do
						settings.should have(2).items
					end

					it "returns an array with correct ids, values and tags" do
						settings.sort_by(&:id).map(&:id).should == ['a', 'c']
						settings.sort_by(&:id).map(&:value).should == ['A', 'C']
						settings.sort_by(&:id).map(&:tags).
								should == [['vowels'], ['consonants']]
					end

					it "deletes the key from the global index" do
						settings
						$redis.smembers(Setting.send(:global_key)).
								should_not include(Setting.send(:generate_key, 'b'))
					end
				end
			end
		end

		describe ".tags" do
			before do
				Setting.persist("a", "A", "vowels")
				Setting.persist("b", "B", "consonants")
				Setting.persist("c", "C", "consonants")
			end

			it "returns all created tags" do
				Setting.tags.should include('vowels', 'consonants')
			end
		end

		describe ".delete" do
			before do
				Setting.persist(name, "I am serialized", "xyz")
				Setting.find(name, true).tags.should == ["xyz"]
			end

			it "deletes the setting under the passed id" do
				Setting.delete(name)
				Setting.find(name).should be_nil
			end

			it "deletes the tags under the passed id" do
				Setting.delete(name)
				$redis.smembers(Setting.send(:generate_tags_key, name)).should be_empty
			end
		end
	end
end
